@@ -3,6 +3,7 @@ use automato::statemachine;
 use futures::executor::block_on;
 use serde::{Deserialize, Serialize};
 use serde_json;
+use std::marker::PhantomData;
 
 #[derive(Serialize, Deserialize)]
 pub struct JobData {}
@@ -32,15 +33,14 @@ statemachine! {
 struct Log {}
 
 #[async_trait]
-impl Observer for Log {
+impl Observer<()> for Log {
     type Error = ();
-    type Context = ();
 }
 
 #[test]
 fn init() {
     let _job = block_on(Job::init(
-        &mut (),
+        (),
         Log {},
         Some("foo".to_string()),
         JobData {},
@@ -51,7 +51,7 @@ fn init() {
 
 #[test]
 fn init_without_id() {
-    let result = block_on(Job::init(&mut (), Log {}, None, JobData {}, QueuedData {}));
+    let result = block_on(Job::init((), Log {}, None, JobData {}, QueuedData {}));
     let err = result.err().unwrap();
     match err {
         InitError::EmptyId => {}
@@ -64,13 +64,12 @@ fn init_with_deferred_id() {
     struct DeferredIdInitLog {}
 
     #[async_trait]
-    impl Observer for DeferredIdInitLog {
+    impl Observer<()> for DeferredIdInitLog {
         type Error = ();
-        type Context = ();
 
         async fn on_init<T: Serialize + Send, U: Serialize + Send>(
             &mut self,
-            _ctx: &mut Self::Context,
+            _ctx: &mut (),
             _id: Option<String>,
             _to: State,
             _data: Option<T>,
@@ -81,7 +80,7 @@ fn init_with_deferred_id() {
     }
 
     let _job = block_on(Job::init(
-        &mut (),
+        (),
         DeferredIdInitLog {},
         None,
         JobData {},
@@ -97,13 +96,12 @@ fn on_init() {
     }
 
     #[async_trait]
-    impl Observer for &mut InitLog {
+    impl Observer<()> for &mut InitLog {
         type Error = ();
-        type Context = ();
 
         async fn on_init<T: Serialize + Send, U: Serialize + Send>(
             &mut self,
-            _ctx: &mut Self::Context,
+            _ctx: &mut (),
             id: Option<String>,
             to: State,
             _data: Option<T>,
@@ -119,7 +117,7 @@ fn on_init() {
     };
 
     let _job = block_on(Job::init(
-        &mut (),
+        (),
         &mut init_log,
         Some("foo".to_string()),
         JobData {},
@@ -136,7 +134,7 @@ fn on_init() {
 #[test]
 fn read_id() {
     let job = block_on(Job::init(
-        &mut (),
+        (),
         Log {},
         Some("foo".to_string()),
         JobData {},
@@ -151,7 +149,7 @@ fn read_id() {
 #[test]
 fn read_data() {
     let job = block_on(Job::init(
-        &mut (),
+        (),
         Log {},
         Some("foo".to_string()),
         JobData {},
@@ -165,14 +163,14 @@ fn read_data() {
 #[test]
 fn transition() {
     let job = block_on(Job::init(
-        &mut (),
+        (),
         Log {},
         Some("foo".to_string()),
         JobData {},
         QueuedData {},
     ))
     .unwrap();
-    let _job = block_on(job.start(&mut (), ProcessingData {})).unwrap();
+    let _job = block_on(job.start((), ProcessingData {})).unwrap();
 }
 
 #[test]
@@ -183,13 +181,12 @@ fn on_transition() {
     }
 
     #[async_trait]
-    impl Observer for &mut TransitionLog {
+    impl Observer<()> for &mut TransitionLog {
         type Error = ();
-        type Context = ();
 
         async fn on_transition<T: Serialize + Send, U: Serialize + Send>(
             &mut self,
-            _ctx: &mut Self::Context,
+            _ctx: &mut (),
             _id: &str,
             from: State,
             to: State,
@@ -208,14 +205,14 @@ fn on_transition() {
     };
 
     let job = block_on(Job::init(
-        &mut (),
+        (),
         &mut transition_log,
         Some("foo".to_string()),
         JobData {},
         QueuedData {},
     ))
     .unwrap();
-    let _job = block_on(job.start(&mut (), ProcessingData {})).unwrap();
+    let _job = block_on(job.start((), ProcessingData {})).unwrap();
 
     match transition_log.from {
         Some(state) => assert_eq!("Queued", state.to_string()),
